package com.jd.dynamic.lib.expv2.evaluators

import android.text.TextUtils
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.lib.expv2.EMPTY_STR
import com.jd.dynamic.lib.expv2.evaluators.base.XPEvaluatorWithAll
import com.jd.dynamic.lib.expv2.needRemovePH
import com.jd.dynamic.lib.expv2.nodes.XPNode
import com.jd.dynamic.lib.expv2.notIllegal
import com.jd.dynamic.lib.expv2.str2BigDecimal

/**
 * @author: xuepanjie1
 * @date:2021/12/29
 * @description: 条件表达式解析器 @{1>2?2:1}
 */

class ConditionEvaluator(engine: DynamicTemplateEngine? = null) : XPEvaluatorWithAll(engine) {
    companion object {
        private const val K_EQUAL = "=="
        private const val K_BIG_OR_EQ = ">="
        private const val K_LESS_OR_EQ = "<="
        private const val K_NOT_EQUAL = "!="
        private const val K_BIG_THAN = ">"
        private const val K_LESS_THAN = "<"
        private const val K_AND = "&&"
        private const val K_OR = "||"
        private const val K_WEN_HAO = '?'
        private const val K_FEN_HAO = ':'
        private const val BOOL_STR = EMPTY_STR
        fun getExp(content: String): String {
            return content.substring(2, content.length - 1)
        }
    }

    override fun eval(node: XPNode): Any? {
        val content = node.parseContent()
        if (null == content || content.length <= 3) {
            return content
        }
        val exp = getExp(content)
        val params = splitConditionParam(exp)
        if (params.isEmpty()) {
            return EMPTY_STR
        }
        if (params.size >= 2) {
            return newJudge(params, node)
        }
        return try {
            judgeExp(exp, node)
        } catch (e: Exception) {
            EMPTY_STR
        }
    }


    private fun newJudge(params: List<String>, node: XPNode): Any? {
        val type = params[0]
        if (K_OR == type) {
            for (param in params) {
                if (K_OR == param || K_AND == param) {
                    continue
                }
                if (judgeExp(param, node)) {
                    return true
                }
            }
            return false
        }
        if (K_AND == type) {
            for (param in params) {
                if (K_OR == param || K_AND == param) {
                    continue
                }
                if (!judgeExp(param, node)) {
                    return false
                }
            }
            return true
        }
        if (3 == params.size) {
            return newTriJudge(params[0], params[1], params[2], node)
        }
        return EMPTY_STR
    }

    private fun newTriJudge(exp: String, val1: String, val2: String, node: XPNode): Any? {
        val params = splitConditionParam(exp)
        if (params.isEmpty()) {
            return EMPTY_STR
        }
        if (params.size >= 2) {
            return if (true == newJudge(params, node)) {
                if (val1.needRemovePH()) node.restore(val1) else val1
            } else {
                if (val2.needRemovePH()) node.restore(val2) else val2
            }
        }
        return if (judgeExp(params[0], node)) {
            if (val1.needRemovePH()) node.restore(val1) else val1
        } else {
            if (val2.needRemovePH()) node.restore(val2) else val2
        }
    }

    private fun judgeExp(exp: String, xpNode: XPNode): Boolean {
        val expParams = createOperatorData(exp)
        return when (expParams[0]) {
            BOOL_STR -> calcBool(expParams[1], xpNode)
            K_EQUAL -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                val result = values.first?.equals(values.second, true)
                return true == result
            }
            K_NOT_EQUAL -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                val result = values.first?.equals(values.second, true)
                return false == result
            }
            K_LESS_THAN -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = str2BigDecimal(values.first as String)
                    val bs = str2BigDecimal(values.second as String)
                    return bf < bs
                }
                return false
            }
            K_LESS_OR_EQ -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = str2BigDecimal(values.first as String)
                    val bs = str2BigDecimal(values.second as String)
                    return bf <= bs
                }
                return false
            }
            K_BIG_THAN -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = str2BigDecimal(values.first as String)
                    val bs = str2BigDecimal(values.second as String)
                    return bf > bs
                }
                return false
            }
            K_BIG_OR_EQ -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = str2BigDecimal(values.first as String)
                    val bs = str2BigDecimal(values.second as String)
                    return bf >= bs
                }
                return false
            }
            else -> false
        }
    }

    private fun originContent(first: String, sec: String, xpNode: XPNode): Pair<String?, String?> {
        var firValue = if (first.needRemovePH()) xpNode.restore(first) else first
        var secValue = if (sec.needRemovePH()) xpNode.restore(sec) else sec
        if (!firValue.notIllegal()) {
            firValue = EMPTY_STR
        }
        if (!secValue.notIllegal()) {
            secValue = EMPTY_STR
        }
        return Pair(firValue?.toString(), secValue?.toString())
    }

    private fun createOperatorData(exp: String): List<String> {
        val result = ArrayList<String>()
        var condition = BOOL_STR
        when {
            exp.contains(K_EQUAL) -> {
                condition = K_EQUAL
            }
            exp.contains(K_NOT_EQUAL) -> {
                condition = K_NOT_EQUAL
            }
            exp.contains(K_BIG_OR_EQ) -> {
                condition = K_BIG_OR_EQ
            }
            exp.contains(K_BIG_THAN) -> {
                condition = K_BIG_THAN
            }
            exp.contains(K_LESS_OR_EQ) -> {
                condition = K_LESS_OR_EQ
            }
            exp.contains(K_LESS_THAN) -> {
                condition = K_LESS_THAN
            }
        }
        if (BOOL_STR != condition) {
            result.add(condition)
            result.addAll(exp.split(condition))
        } else {
            result.add(BOOL_STR)
            result.add(exp)
        }
        return result
    }

    private fun calcBool(exp: String, xpNode: XPNode): Boolean {
        val finalExp = if (exp.needRemovePH()) xpNode.restore(exp) else exp
        if (finalExp is String) {
            return "true".equals(finalExp, true)
        }
        if (finalExp is Boolean) {
            return finalExp
        }
        return false
    }

    private fun splitConditionParam(origin: String): List<String> {
        val result = ArrayList<String>()
        val whList = origin.split(K_WEN_HAO)

        if (1 == whList.size) {
            val andL = origin.split(K_AND)
            if (2 <= andL.size) {
                result.add(K_AND)
                result.addAll(andL)
                return result
            }
            val orL = origin.split(K_OR)
            if (orL.size >= 2) {
                result.add(K_OR)
                result.addAll(orL)
                return result
            }
            result.add(origin)
        } else if (2 == whList.size) {
            val fh = whList[1].split(K_FEN_HAO)
            if (2 == fh.size) {
                result.add(whList[0])
                result.addAll(fh)
            }
        }
        return result
    }
}