package com.jd.dynamic.lib.expv2.evaluators

import android.view.View
import com.jd.dynamic.DYConstants
import com.jd.dynamic.base.CachePool
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.lib.ab.DYABConfigUtil
import com.jd.dynamic.lib.expv2.*
import com.jd.dynamic.lib.expv2.nodes.PathNode
import com.jd.dynamic.lib.expv2.nodes.XPNode
import com.jd.dynamic.lib.expv2.evaluators.base.XPEvaluatorWithAll
import com.jd.dynamic.lib.expv2.evaluators.util.ResultContainer
import com.jd.dynamic.lib.expv2.evaluators.util.TakeValueHelper
import org.json.JSONArray
import org.json.JSONObject

/**
 * @author: xuepanjie1
 * @date:2021/12/28
 * @description: 取值表达式解析器
 * e.g. ${info.test}   /  ${info.test\[1]} ${info.test\[${hhh.dd}]}   /
 * ${info.test.count.$count} count这里最好还是加上$   /  ${fun(ddd)}
 */

class TakeValueEvaluator(
    engine: DynamicTemplateEngine? = null,
    obj: Any? = null,
    view: View? = null
) : XPEvaluatorWithAll(engine, obj, view) {
    companion object {
        private const val K_LENGTH = "length"

        const val K_OBJECT = "object"
        const val K_THIS = "targetView"
        const val K_CACHE = "cacheKey"
        const val K_POINT = '.'
        const val K_COUNT = "count"
        const val K_COUNT_DOLLAR = "\$count"
        const val K_CONTAINER = "container"
        const val K_EST = "estimatedSize"

        fun dealEST(key: Any?, type: Any?, engine: DynamicTemplateEngine?) : Int? {
            if (K_EST == key) {
                return if (DYConstants.DY_HEIGHT == type) {
                    engine?.containerHeight
                } else if (DYConstants.DY_WIDTH == type) {
                    engine?.containerWidth
                } else {
                    return DYConstants.DY_INT_ZERO
                }
            }
            return DYConstants.DY_INT_ZERO
        }

        fun isLength(key: String) : Boolean {
            return K_LENGTH.equals(key, true)
        }

        fun checkIsRange(str: String): Boolean {
            val lIndex = str.indexOf(SYMBOL_MIDDLE_BRACKET_L)
            val cIndex = str.indexOf(SYMBOL_COLON)
            val rIndex = str.indexOf(SYMBOL_MIDDLE_BRACKET_R)
            return lIndex in 0 until cIndex && rIndex > cIndex
        }

        fun str2range(str: String): Pair<String, Pair<Int, Int>> {
            val lIndex = str.indexOf(SYMBOL_MIDDLE_BRACKET_L)
            val cIndex = str.indexOf(SYMBOL_COLON)
            val rIndex = str.indexOf(SYMBOL_MIDDLE_BRACKET_R)
            val key = str.substring(0, lIndex)
            val start = if (cIndex - lIndex == 1) 0 else str2Int(str.substring(lIndex + 1, cIndex))
            val end = if (rIndex - cIndex == 1) Int.MAX_VALUE else str2Int(str.substring(cIndex + 1, rIndex))
            if (end <= start) {
                return Pair(EMPTY_STR, Pair(INVALID_POS, INVALID_POS))
            }
            return Pair(key, Pair(start, end))
        }

        fun str2Int(str: String?): Int {
            if (null == str) {
                return Int.MIN_VALUE
            }
            var value = str
            return try {
                if (str.contains(SYMBOL_MIDDLE_BRACKET_L)) {
                    value = str.substring(0, str.length - 1)
                }
                value.toFloat().toInt()
            } catch (e: Exception) {
                Int.MIN_VALUE
            }
        }

        fun str2Str(str: String): Pair<String, String> {
            return try {
                Pair(
                    str.substring(0, str.indexOf(SYMBOL_MIDDLE_BRACKET_L)), str.substring(
                        str.indexOf(
                            SYMBOL_MIDDLE_BRACKET_L
                        ) + 1, str.indexOf(SYMBOL_MIDDLE_BRACKET_R)
                    )
                )
            } catch (e: Exception) {
                Pair(EMPTY_STR, EMPTY_STR)
            }
        }

        fun takeValueFromArray(array: JSONArray, strs: Any?, node: XPNode): Any? {
            if (null == strs || strs !is String) {
                return null
            }
            var finalStr: Any? = strs
            if (strs.needRemovePH()) {
                finalStr = node.restore(strs)
            }
            val index = str2Int(finalStr as? String)
            if (Int.MIN_VALUE == index) {
                return null
            }
            if (index >= 0 && index < array.length()) {
                return array[index]
            }
            return null
        }

        // [index:count] index为负值则是从后往前取，这里的关键是计算出start和end即可，都是正数那么end=start+count
        fun takeArrayFromArray(array: JSONArray, keyAndRange: Pair<Int, Int>): Any? {
            val len = array.length()
            var start = keyAndRange.first
            var count = keyAndRange.second

            if (count <= 0) {
                return null
            }

            if (Int.MAX_VALUE == count) {
                count = if (start < 0) {
                    -start
                } else {
                    len - start
                }
            }

            if (count > array.length()) {
                return null
            }

            if (start < 0) {
                start += len
                if (start < 0) {
                    return null
                }
            }

            val end = start + count
            if (end > len) {
                return null
            }
            val result = JSONArray()
            for (i in start until end) {
                result.put(array.get(i))
            }
            return result
        }

        fun takeValueFromJSONArray(obj: Any?, exp: Any, node: XPNode) : Any? {
            var objInner = obj
            val keyOut = exp
            if (keyOut is String && keyOut.contains(SYMBOL_MIDDLE_BRACKET_L)) {
                if (!checkIsRange(keyOut)) {
                    val result = str2Str(keyOut)
                    if (result.second.isEmpty()) {
                        return objInner
                    }
                    if (result.first.isEmpty() && objInner is JSONArray) {
                        return takeValueFromArray(objInner, result.second, node)
                    }
                    if (objInner is JSONObject) {
                        objInner = objInner.get(result.first)
                    }
                    if (objInner is JSONArray) {
                        objInner = takeValueFromArray(objInner, result.second, node)
                    }
                    return objInner
                } else {
                    val pairRange = str2range(keyOut)
                    if (EMPTY_STR == pairRange.first && objInner is JSONArray) {
                        return takeArrayFromArray(objInner, pairRange.second)
                    }
                    if (objInner is JSONObject) {
                        objInner = objInner.get(pairRange.first)
                    }
                    if (objInner is JSONArray) {
                        objInner = takeArrayFromArray(objInner, pairRange.second)
                    }
                    return objInner
                }
            }
            return objInner
        }
    }

    override fun eval(node: XPNode): Any? {
        val content = node.parseContent()
        if (null == content || content.length <= 3 || node !is PathNode) {
            return content
        }

        val exp = ConditionEvaluator.getExp(content)
        return takeValue(obj, exp, node)
    }

    private fun createExpNode(origin: String) : List<Any> {
        var str = ""
        val ccResult = ArrayList<String>()
        for (cc in origin.toCharArray()) {
            if (K_POINT == cc) {
                if ("" != str) {
                    ccResult.add(str)
                }
                str = ""
            } else {
                str += cc
            }
        }
        if ("" != str) {
            ccResult.add(str)
        }
        return ccResult
    }
    
    private fun takeValue(obj: Any?, exp: String, node: XPNode): Any? {
        if (!exp.notIllegal()) {
            return null
        }

        val objs = exp.split(K_POINT)
        if (obj is JSONArray) {
            return takeValueFromJSONArray(obj, exp, node)
        }

        return takeValueFromObj(obj, objs, node)
    }

    private fun takeValueFromObj(obj: Any?, strs: List<Any>, node: XPNode): Any? {
        var objInner: Any? = if (obj !is View) obj else null
        val iter = strs.iterator()
        val enter = System.nanoTime()
        while (iter.hasNext()) {
            val current = System.nanoTime()

            var key = iter.next()

            if (key is String && key.needRemovePH()) {
                val res = node.restore(key)
                if (null == res) {
                    continue
                } else {
                    key = res
                }
            }

            if (objInner is String) {
                if (FLAG_B == objInner.containBOrM()) {
                    try {
                        objInner = JSONObject(objInner)
                        DYConstants.DYLog("str 2 json : ${System.nanoTime() - current}")
                    } catch (e: Exception) {
                    }
                }
            }
            if (key is ResultContainer) {
                objInner = key.value
                continue
            }
            if (key is JSONObject || key is JSONArray) {
                objInner = key
                continue
            }
            if (key is String) {
                if (FLAG_B == key.containBOrM()) {
                    try {
                        objInner = JSONObject(key)
                        DYConstants.DYLog(" key is json : ${System.nanoTime() - current}")
                        continue
                    } catch (e: Exception) {
                    }
                }
                if (key.contains(SYMBOL_MIDDLE_BRACKET_L)) {
                    var result: Pair<String, String>? = null
                    result = str2Str(key)
                    if (result.first.isEmpty() || result.second.isEmpty()) {
                        return objInner
                    }
                    if (objInner !is JSONObject) {
                        return objInner
                    }
                    objInner = objInner.opt(result.first)
                    if (objInner is JSONArray) {
                        objInner = takeValueFromArray(objInner, result.second, node)
                    }
                    if (null == objInner) {
                        return objInner
                    }
                    continue
                }
                if (isLength(key)) {
                    if (objInner is String) {
                        return objInner.length
                    }
                    if (objInner is JSONObject) {
                        try {
                            val lengthTemp = objInner.opt(key)
                            if (null != lengthTemp) {
                                objInner = lengthTemp
                                continue
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }

                }
                val count = isCount(key)
                if (count.isNotEmpty()) {
                    if (K_COUNT == count) {
                        if (objInner is JSONObject) {
                            try {
                                if (null != objInner.opt(key)) {
                                    objInner = objInner.opt(key)
                                    continue
                                } else {
                                    return objInner.length()
                                }
                            } catch (e: Exception) {
                                e.printStackTrace()
                            }
                        }
                    }

                    if (K_COUNT_DOLLAR == count) {
                        if (objInner is JSONObject) {
                            return objInner.length()
                        }
                    }

                    return when (objInner) {
                        is JSONArray -> {
                            objInner.length()
                        }
                        is JSONObject -> {
                            objInner.length()
                        }
                        is String -> {
                            objInner.length
                        }
                        else -> {
                            objInner
                        }
                    }
                }
                if (K_CACHE.equals(key, true)) {
                    if (iter.hasNext()) {
                        var nextKey = iter.next()
                        if (nextKey is String && nextKey.needRemovePH()) {
                            val value = node.restore(nextKey)
                            if (null == value) {
                                return null
                            } else {
                                nextKey = value
                            }
                        }
                        if (nextKey is String) {
                            objInner = if (DYABConfigUtil.getInstance().isCacheUseObj) {
                                engine?.cachePool?.getDataObj(nextKey)
                            } else {
                                engine?.cachePool?.getData(nextKey)
                            }
                            if (null == objInner) {
                                return objInner
                            }
                        }
                    }
                    continue
                }

                if (K_THIS.equals(key, true)) {
                    objInner = view
                    continue
                }
                if (K_CONTAINER == key) {
                    var first:Any? = null
                    var sec:Any? = null

                    if (iter.hasNext()) {
                        first = iter.next()
                        if (iter.hasNext()) {
                            sec = iter.next()
                        }
                    }

                    return dealEST(first, sec, engine)
                }
                if (K_OBJECT.equals(key, true)) {
                    objInner = obj
                    continue
                }
                objInner = if (objInner is JSONObject) {
                    objInner.opt(key)
                } else {
                    val attrMethod = CachePool.getMethod(key)
                    if (attrMethod != null) {
                        val result = TakeValueHelper.takeValueFromMethod(objInner,
                            attrMethod, engine)
                        DYConstants.DYLog("result is : $result")
                        result
                    } else {
                        null
                    }
                }
                DYConstants.DYLog("after opt key : ${System.nanoTime() - current}" +
                        " key is : $key")
            }
            if (null == objInner) {
                return objInner
            }
        }
        DYConstants.DYLog("return time ${System.nanoTime() - enter} obj inner: $objInner")
        return objInner
    }

    private fun isCount(key: String): String {
        if (K_COUNT.equals(key, true)) {
            return K_COUNT
        }
        if (K_COUNT_DOLLAR.equals(key, true)) {
            return K_COUNT_DOLLAR
        }
        return EMPTY_STR
    }

}