package com.jd.dynamic.lib.expv2.nodes

import android.text.TextUtils
import android.view.View
import com.jd.dynamic.base.DynamicTemplateEngine
import com.jd.dynamic.lib.expv2.EMPTY_STR
import com.jd.dynamic.lib.expv2.SYMBOL_BIG_BRACKET_R
import com.jd.dynamic.lib.expv2.needRemovePH
import com.jd.dynamic.lib.expv2.notIllegal
import java.lang.Exception
import java.math.BigDecimal

/**
 * @author: xuepanjie1
 * @date:2021/12/28
 * @description: 条件表达式节点
 */

class ConditionNode(flag: String) : XPRootNode(flag) {
    override fun isMatch(tail: String): Boolean {
        this.tail = tail
        return tail == SYMBOL_BIG_BRACKET_R.toString()
    }

    companion object {
        private const val K_EQUAL = "=="
        private const val K_BIG_OR_EQ = ">="
        private const val K_LESS_OR_EQ = "<="
        private const val K_NOT_EQUAL = "!="
        private const val K_BIG_THAN = ">"
        private const val K_LESS_THAN = "<"
        private const val K_AND = "&&"
        private const val K_OR = "||"
        private const val K_WEN_HAO = '?'
        private const val K_FEN_HAO = ':'
        private const val BOOL_STR = EMPTY_STR
        fun getExp(content: String): String {
            return content.substring(2, content.length - 1)
        }
    }

    override fun evalResultInner(
        obj: Any?,
        engine: DynamicTemplateEngine?,
        targetView: View?
    ): Any? {
        val content = super.evalResultInner(obj, engine, targetView)
        if (content !is String || content.length <= 3) {
            return content
        }
        val exp = getExp(content)
        val params = splitConditionParam(exp)
        if (params.isEmpty()) {
            return null
        }
        if (2 == params.size) {
            return calcAndOr(params, this)
        }
        if (3 == params.size) {
            val result = calcTrinocularOperation(params, this)
            return if (!result.notIllegal()) EMPTY_STR else result
        }
        return try {
            judgeExp(exp, this)
        } catch (e: Exception) {
            EMPTY_STR
        }
    }

    fun eval(node: XPNode): Any? {
        val content = node.parseContent()
        if (null == content || content.length <= 3) {
            return content
        }
        val exp = getExp(content)
        val params = splitConditionParam(exp)
        if (params.isEmpty()) {
            return null
        }
        if (2 == params.size) {
            return calcAndOr(params, node)
        }
        if (3 == params.size) {
            val result = calcTrinocularOperation(params, node)
            return if (!result.notIllegal()) EMPTY_STR else result
        }
        return try {
            judgeExp(exp, node)
        } catch (e: Exception) {
            EMPTY_STR
        }
    }


    private fun calcTrinocularOperation(params: List<String>, node: XPNode): Any? {
        val val1 = if (params[1].needRemovePH()) node.restore(params[1]) else params[1]
        val val2 = if (params[2].needRemovePH()) node.restore(params[2]) else params[2]
        val exp = params[0]
        return run {
            val result = if (calcAndOr(splitConditionParam(exp), node)) val1 else val2
            if (!result.notIllegal()) {
                EMPTY_STR
            } else {
                result
            }
        }
    }

    private fun calcAndOr(params: List<String>, node: XPNode): Boolean {
        val type = params[0]
        if (K_OR == type) {
            return try {
                judgeExp(params[1], node) || judgeExp(params[2], node)
            } catch (e: Exception) {
                false
            }
        }
        if (K_AND == type) {
            return try {
                return judgeExp(params[1], node) && judgeExp(params[2], node)
            } catch (e: Exception) {
                false
            }
        }
        return try {
            return judgeExp(params[0], node)
        } catch (e: Exception) {
            false
        }
    }

    private fun judgeExp(exp: String, xpNode: XPNode): Boolean {
        val expParams = createOperatorData(exp)
        return when (expParams[0]) {
            BOOL_STR -> calcBool(expParams[1], xpNode)
            K_EQUAL -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                val result = values.first?.equals(values.second, true)
                return true == result
            }
            K_NOT_EQUAL -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                return !TextUtils.equals(values.first, values.second)
            }
            K_LESS_THAN -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = BigDecimal(values.first)
                    val bs = BigDecimal(values.second)
                    return bf < bs
                }
                return false
            }
            K_LESS_OR_EQ -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = BigDecimal(values.first)
                    val bs = BigDecimal(values.second)
                    return bf <= bs
                }
                return false
            }
            K_BIG_THAN -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = BigDecimal(values.first)
                    val bs = BigDecimal(values.second)
                    return bf > bs
                }
                return false
            }
            K_BIG_OR_EQ -> {
                val values = originContent(expParams[1], expParams[2], xpNode)
                if (values.first is String && values.second is String) {
                    val bf = BigDecimal(values.first)
                    val bs = BigDecimal(values.second)
                    return bf >= bs
                }
                return false
            }
            else -> false
        }
    }

    private fun originContent(first: String, sec: String, xpNode: XPNode): Pair<String?, String?> {
        var firValue = if (first.needRemovePH()) xpNode.restore(first) else first
        var secValue = if (sec.needRemovePH()) xpNode.restore(sec) else sec
        if (!firValue.notIllegal()) {
            firValue = EMPTY_STR
        }
        if (!secValue.notIllegal()) {
            secValue = EMPTY_STR
        }
        return Pair(firValue?.toString(), secValue?.toString())
    }

    private fun createOperatorData(exp: String): List<String> {
        val result = ArrayList<String>()
        var condition = BOOL_STR
        when {
            exp.contains(K_EQUAL) -> {
                condition = K_EQUAL
            }
            exp.contains(K_NOT_EQUAL) -> {
                condition = K_NOT_EQUAL
            }
            exp.contains(K_BIG_OR_EQ) -> {
                condition = K_BIG_OR_EQ
            }
            exp.contains(K_BIG_THAN) -> {
                condition = K_BIG_THAN
            }
            exp.contains(K_LESS_OR_EQ) -> {
                condition = K_LESS_OR_EQ
            }
            exp.contains(K_LESS_THAN) -> {
                condition = K_LESS_THAN
            }
        }
        if (BOOL_STR != condition) {
            result.add(condition)
            result.addAll(exp.split(condition))
        } else {
            result.add(BOOL_STR)
            result.add(exp)
        }
        return result
    }

    private fun calcBool(exp: String, xpNode: XPNode): Boolean {
        val finalExp = if (exp.needRemovePH()) xpNode.restore(exp) else exp
        if (finalExp is String) {
            return "true".equals(finalExp, true)
        }
        if (finalExp is Boolean) {
            return finalExp
        }
        return false
    }

    private fun splitConditionParam(origin: String): List<String> {
        val result = ArrayList<String>()
        val whList = origin.split(K_WEN_HAO)

        if (1 == whList.size) {
            val andL = origin.split(K_AND)
            if (2 == andL.size) {
                result.add(K_AND)
                result.addAll(andL)
                return result
            }
            val orL = origin.split(K_OR)
            if (2 == orL.size) {
                result.add(K_OR)
                result.addAll(orL)
                return result
            }
            result.add(origin)
        } else if (2 == whList.size) {
            val fh = whList[1].split(K_FEN_HAO)
            if (2 == fh.size) {
                result.add(whList[0])
                result.addAll(fh)
            }
        }
        return result
    }
}